import { NgClass, NgFor, NgIf } from '@angular/common';
import {
  Component,
  EventEmitter,
  OnInit,
  Output,
  ViewContainerRef,
} from '@angular/core';
import { FormsModule } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import * as L from 'leaflet';
import 'leaflet.markercluster';
import { Pin } from '../../model/Pin';
import { AuthService } from '../../services/auth/auth.service';
import { AutocompleteService } from '../../services/auto-complete/auto-complete.service';
import { IntroService } from '../../services/intro/intro.service';
import { MapReloadService } from '../../services/map-reload/map-reload.service';
import { ModalService } from '../../services/modal/modal.service';
import { PinService } from '../../services/pin/pin.service';
import { PinMarkerComponent } from '../pin-marker/pin-marker.component';

@Component({
  selector: 'app-leaflet-map',
  templateUrl: './leaflet-map.component.html',
  standalone: true,
  imports: [NgFor, FormsModule, NgIf, NgClass],
})
export class LeafletMapComponent implements OnInit {
  private map!: L.Map;
  private markersMap: { [key: string]: L.Marker } = {};
  private markerClusterGroup!: L.MarkerClusterGroup;
  private allPins: Pin[] = [];
  private pinCountries: { [pinId: string]: string } = {};
  private contextMenu: L.Popup | null = null;
  private user_id: string = '';

  availableCountries: string[] = [];
  availablePersons: string[] = [];
  selectedCountry: string = '__all__';
  selectedPerson: string = '__all__';
  isFiltersVisible: boolean = false;

  fileNames: string[] = [];
  @Output() filesSelected = new EventEmitter<FileList>();

  constructor(
    private viewContainerRef: ViewContainerRef,
    private pinsService: PinService,
    private route: ActivatedRoute,
    private router: Router,
    private modalService: ModalService,
    private mapReloadService: MapReloadService,
    private introService: IntroService,
    private autocompleteService: AutocompleteService,
    private authService: AuthService
  ) {
    this.user_id = this.authService.getUserId();
  }

  ngOnInit(): void {
    this.initializeMap();

    this.mapReloadService.reload$.subscribe(() => {
      this.loadPins(); // recharge les pins quand demandé
    });

    this.route.params.subscribe((params) => {
      if (params['tutorial'] === 'true') {
        this.introService.startIntro();
      }
    });

    this.route.queryParams.subscribe((params) => {
      const pinId = params['pin'];

      if (pinId) {
        const marker = this.markersMap[pinId];
        if (marker) {
          marker.openPopup();
          const latlng = marker.getLatLng();
          const zoom = this.map.getZoom();
          const offsetLat = 0.05 / Math.pow(2, zoom - 10);
          this.map.setView(L.latLng(latlng.lat + offsetLat, latlng.lng), zoom);
        }
      }
    });
  }

  private initializeMap(): void {
    this.map = L.map('map', {
      maxBounds: L.latLngBounds(L.latLng(-90, -180), L.latLng(90, 180)),
      maxBoundsViscosity: 1.0,
      minZoom: 2,
    }).setView([46.603354, 1.888334], 6);

    L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
      attribution: '',
    }).addTo(this.map);
    this.map.attributionControl.setPrefix('');

    // Initialiser le groupe de clusters
    this.markerClusterGroup = window.L.markerClusterGroup();
    this.map.addLayer(this.markerClusterGroup);

    // Ajouter l'événement de clic droit sur la carte
    this.map.on('contextmenu', (e: L.LeafletMouseEvent) => {
      e.originalEvent.preventDefault();
      this.showContextMenu(e.latlng);
    });

    // Fermer le menu contextuel lors d'un clic gauche sur la carte
    this.map.on('click', () => {
      if (this.contextMenu) {
        this.map.closePopup(this.contextMenu);
        this.contextMenu = null;
      }
    });

    this.pinsService.getPins().subscribe((pins: Pin[]) => {
      this.allPins = pins;
      this.extractPersons(pins);

      this.renderPins();
      this.loadCountriesForFiltrers(pins);
    });
  }

  private showContextMenu(latlng: L.LatLng): void {
    // Fermer le menu contextuel existant s'il y en a un
    if (this.contextMenu) {
      this.map.closePopup(this.contextMenu);
    }

    // Créer le contenu du menu contextuel
    const menuContent = document.createElement('div');
    menuContent.className =
      'bg-white dark:bg-gray-800 rounded-lg shadow-lg overflow-hidden';

    const addPinButton = document.createElement('button');
    addPinButton.className =
      'w-full px-4 py-2.5 text-sm font-medium text-gray-900 dark:text-white hover:bg-gray-100 dark:hover:bg-gray-700 focus:outline-none focus:ring-2 focus:ring-blue-500 dark:focus:ring-blue-600 flex items-center gap-2';

    // Ajouter l'icône de pin
    const pinIcon = document.createElement('span');
    pinIcon.innerHTML = `
      <svg class="w-4 h-4" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24" stroke-width="1.5" stroke="currentColor">
        <path stroke-linecap="round" stroke-linejoin="round" d="M15 10.5a3 3 0 1 1-6 0 3 3 0 0 1 6 0Z" />
        <path stroke-linecap="round" stroke-linejoin="round" d="M19.5 10.5c0 7.142-7.5 11.25-7.5 11.25S4.5 17.642 4.5 10.5a7.5 7.5 0 1 1 15 0Z" />
      </svg>
    `;

    const buttonText = document.createElement('span');
    buttonText.textContent = 'Ajouter un pin ici';

    addPinButton.appendChild(pinIcon);
    addPinButton.appendChild(buttonText);

    addPinButton.onclick = () => {
      this.addPinAtLocation(latlng);
      if (this.contextMenu) {
        this.map.closePopup(this.contextMenu);
        this.contextMenu = null;
      }
    };

    menuContent.appendChild(addPinButton);

    // Créer et afficher le popup
    this.contextMenu = L.popup({
      closeButton: false,
      className: 'context-menu-popup',
      maxWidth: 200,
      offset: [0, -10],
    })
      .setLatLng(latlng)
      .setContent(menuContent)
      .addTo(this.map);
  }

  private async addPinAtLocation(latlng: L.LatLng): Promise<void> {
    try {
      // Obtenir l'adresse à partir des coordonnées
      const address = await this.autocompleteService
        .getAddressFromCoordinates(latlng.lat, latlng.lng)
        .toPromise();

      // Ouvrir la modal avec les données pré-remplies
      this.modalService.openModal('add-pin-modal', [], {
        location: address?.display_name || '',
        complete_address: address?.display_name || '',
        coordinates: [latlng.lat, latlng.lng],
      });
    } catch (error) {
      console.error("Erreur lors de la récupération de l'adresse:", error);
      // En cas d'erreur, ouvrir la modal avec juste les coordonnées
      this.modalService.openModal('add-pin-modal', [], {
        location: `${latlng.lat}, ${latlng.lng}`,
        complete_address: `${latlng.lat}, ${latlng.lng}`,
        coordinates: [latlng.lat, latlng.lng],
      });
    }
  }

  private loadCountriesForFiltrers(pins: Pin[]): void {
    const countrySet = new Set<string>();
    const requests = pins.map((pin: Pin) => {
      const country = this.extractLastFromDisplayName(pin.complete_address);
      if (country) {
        this.pinCountries[pin.id] = country;
        countrySet.add(country);
      }
    });

    Promise.all(requests).then(() => {
      this.availableCountries = Array.from(countrySet).sort();
    });
  }

  private extractLastFromDisplayName(displayName: string): string {
    if (!displayName) return '';
    const parts = displayName.split(',');
    return parts[parts.length - 1].trim();
  }

  private extractPersons(pins: Pin[]): void {
    const personsSet = new Set<string>();

    // Pour chaque pin, récupérer ses partages
    pins.forEach((pin) => {
      if (!pin.is_poi) {
        this.pinsService.getPinShares(pin.id).subscribe((response: any) => {
          if (response && response.shares) {
            response.shares.forEach((share: any) => {
              personsSet.add(share.username);
            });
            this.availablePersons = Array.from(personsSet).sort();
          }
        });
      }
    });
  }

  onCountryChange(country: string) {
    this.selectedCountry = country;
    this.renderPins();
  }

  onPersonChange(person: string) {
    this.selectedPerson = person;
    this.renderPins();
  }

  private renderPins(): void {
    // Remove existing markers
    Object.values(this.markersMap).forEach((marker) =>
      this.map.removeLayer(marker)
    );
    this.markerClusterGroup.clearLayers();
    this.markersMap = {};

    const visitedIcon = this.createDivIcon(`
      <svg class="w-6 h-6 text-gray-800" xmlns="http://www.w3.org/2000/svg" fill="currentColor" viewBox="0 0 24 24">
        <path fill-rule="evenodd" d="M11.906 1.994a8.002 8.002 0 0 1 8.09 8.421 7.996 7.996 0 0 1-1.297 3.957.996.996 0 0 1-.133.204l-.108.129c-.178.243-.37.477-.573.699l-5.112 6.224a1 1 0 0 1-1.545 0L5.982 15.26l-.002-.002a18.146 18.146 0 0 1-.309-.38l-.133-.163a.999.999 0 0 1-.13-.202 7.995 7.995 0 0 1 6.498-12.518ZM15 9.997a3 3 0 1 1-5.999 0 3 3 0 0 1 5.999 0Z" clip-rule="evenodd"/>
      </svg>
    `);

    const shareIcon = this.createDivIcon(`
      <svg class="w-6 h-6 text-pink-600" xmlns="http://www.w3.org/2000/svg" fill="currentColor" viewBox="0 0 24 24">
        <path fill-rule="evenodd" d="M11.906 1.994a8.002 8.002 0 0 1 8.09 8.421 7.996 7.996 0 0 1-1.297 3.957.996.996 0 0 1-.133.204l-.108.129c-.178.243-.37.477-.573.699l-5.112 6.224a1 1 0 0 1-1.545 0L5.982 15.26l-.002-.002a18.146 18.146 0 0 1-.309-.38l-.133-.163a.999.999 0 0 1-.13-.202 7.995 7.995 0 0 1 6.498-12.518ZM15 9.997a3 3 0 1 1-5.999 0 3 3 0 0 1 5.999 0Z" clip-rule="evenodd"/>
      </svg>
    `);

    const poiPin = this.createDivIcon(`
    <svg class="w-6 h-6 text-yellow-500" xmlns="http://www.w3.org/2000/svg" fill="currentColor" viewBox="0 0 24 24">
      <path 
        d="M12 .587l3.668 7.431 8.2 1.191-5.934 5.782 1.401 8.169L12 18.897l-7.335 3.863 1.401-8.169L.132 9.209l8.2-1.191z"
        stroke="black"
        stroke-width="1"
      />
    </svg>
  `);

    const filteredPins = this.allPins.filter((pin) => {
      const pinCountry = this.pinCountries[pin.id];
      const matchesCountry =
        this.selectedCountry === '__all__'
          ? true
          : pinCountry === this.selectedCountry;
      const matchesPerson =
        this.selectedPerson === '__all__'
          ? true
          : this.selectedPerson === '__none__'
          ? !pin.description?.match(/@\w+/)
          : pin.description?.includes(`@${this.selectedPerson}`);

      return matchesCountry && matchesPerson;
    });

    filteredPins.forEach((pin) => {
      let markerIcon;
      if (pin.is_poi) {
        markerIcon = poiPin;
      } else if (pin.user_id !== this.user_id) {
        markerIcon = shareIcon;
      } else {
        markerIcon = visitedIcon;
      }
      const marker = L.marker(pin.location as [number, number], {
        icon: markerIcon,
      });

      marker.on('popupclose', () => {
        if (this.router.url.includes('/map')) {
          this.router.navigate(['/map']);
        }
      });

      marker.on('popupopen', () => {
        this.router.navigate(['/map'], { queryParams: { pin: pin.id } });
      });

      const popupDiv = document.createElement('div');
      const componentRef =
        this.viewContainerRef.createComponent(PinMarkerComponent);
      componentRef.instance.pin = pin;
      componentRef.instance.marker = marker;
      popupDiv.appendChild(componentRef.location.nativeElement);

      marker.bindPopup(popupDiv, {
        closeButton: false,
        minWidth: 300,
        maxWidth: 400,
        maxHeight: 400,
        className: 'custom-popup-fixed-size',
      });

      this.markersMap[pin.id] = marker;

      this.markerClusterGroup.addLayer(marker);
    });

    this.markerClusterGroup.refreshClusters();

    // Vérifier si un pin est sélectionné
    const pinId = this.route.snapshot.queryParamMap.get('pin');

    // Ajuster la vue si un pays est sélectionné
    if (
      this.selectedCountry !== '__all__' &&
      filteredPins.length > 0 &&
      !pinId
    ) {
      const bounds = L.latLngBounds(
        filteredPins.map((pin) => pin.location as [number, number])
      );
      this.map.fitBounds(bounds, {
        padding: [50, 50], // Ajoute un peu d'espace autour des marqueurs
        maxZoom: 10, // Limite le zoom maximum pour garder une vue d'ensemble
        animate: true, // Active l'animation
        duration: 1.5, // Durée de l'animation en secondes
      });
    }

    if (pinId) {
      const marker = this.markersMap[pinId];
      if (marker) {
        const latlng = marker.getLatLng();
        this.map.setView(L.latLng(latlng.lat, latlng.lng), 20);
        setTimeout(() => {
          marker.openPopup();
        }, 100);
      }
    }
  }

  private createDivIcon(htmlContent: string): L.DivIcon {
    return L.divIcon({
      html: htmlContent,
      className: 'text-2xl text-blue-500',
      iconSize: [24, 24],
      iconAnchor: [12, 24],
      popupAnchor: [0, -24],
    });
  }

  public loadPins(): void {
    this.pinsService.getPins().subscribe((pins: Pin[]) => {
      // Supprimer du body toutes les divs confirm-modal-* / share-modal-* / edit-pin-popup-*
      const modals = document.querySelectorAll(
        'div[id^="confirm-modal-"], div[id^="share-modal-"], div[id^="edit-pin-popup-"]'
      );
      modals.forEach((modal) => {
        modal.remove();
      });

      this.allPins = pins;
      this.extractPersons(pins);
      this.renderPins(); // Afficher d'abord les pins sans les filtres

      this.loadCountriesForFiltrers(pins); // Ensuite, charger les pays en arrière-plan
    });
  }

  async onFilesSelected(event: Event): Promise<void> {
    const input = event.target as HTMLInputElement;
    if (input.files && input.files.length > 0) {
      this.fileNames = Array.from(input.files).map((f) => f.name);
      this.filesSelected.emit(input.files);
      this.modalService.openModal('add-pin-modal');
    }
  }

  onDragOver(event: DragEvent) {
    event.preventDefault();
  }

  onDrop(event: DragEvent) {
    event.preventDefault();
    if (event.dataTransfer && event.dataTransfer.files.length > 0) {
      const files = event.dataTransfer.files;
      this.filesSelected.emit(files);
      this.modalService.openModal('add-pin-modal', Array.from(files));
    }
  }

  toggleFilters(): void {
    this.isFiltersVisible = !this.isFiltersVisible;
  }

  isDesktop(): boolean {
    return window.innerWidth >= 768;
  }
}
