import { CommonModule } from '@angular/common';
import { Component, ElementRef, ViewChild, AfterViewInit } from '@angular/core';
import { Pin } from '../../model/Pin';
import { PinService } from '../../services/pin/pin.service';
import { RouterModule } from '@angular/router';
import { ActivatedRoute } from '@angular/router';
import { UserService } from '../../services/user/user.service';
import { ImageService } from '../../services/image/image.service';
import { DomSanitizer, SafeUrl } from '@angular/platform-browser';
import { Router } from '@angular/router';
import { CookiesService } from '../../services/cookies/cookies.service';

@Component({
  selector: 'app-pin-detail',
  templateUrl: './pin-detail.component.html',
  styleUrls: ['./pin-detail.component.css'],
  standalone: true,
  imports: [CommonModule, RouterModule],
})
export class PinDetailComponent {
  pin!: Pin;
  username: string = '';
  imageUrls: SafeUrl[] = [];
  imagesLoaded = false;
  sharedUsers: {
    user_id: string;
    username: string;
    can_edit: boolean;
    can_delete: boolean;
  }[] = [];
  expandedDescription = false;
  currentIndex: number = 0;

  username_session: string = '';

  @ViewChild('desc') descriptionDiv!: ElementRef<HTMLDivElement>;
  showToggleButton = false;

  constructor(
    private pinService: PinService,
    private route: ActivatedRoute,
    private userService: UserService,
    private imageService: ImageService,
    private sanitizer: DomSanitizer,
    private router: Router,
    private cookiesService: CookiesService
  ) {}

  ngOnInit(): void {
    const pinId = this.route.snapshot.paramMap.get('id');
    this.username_session = this.cookiesService.getUsername() || '';

    if (pinId) {
      this.pinService.getPinById(pinId).subscribe({
        next: (data) => {
          this.pin = data;
          this.loadUsername(data.user_id);
          this.loadImages();
        },
        error: (err) =>
          console.error('Erreur lors du chargement du pin :', err),
      });

      this.pinService.getSharedUsersForPin(pinId).subscribe({
        next: (shares) => {
          this.sharedUsers = shares.map((share) => ({
            user_id: share.user_id,
            username: share.username,
            can_edit: share.can_edit,
            can_delete: share.can_delete,
          }));
        },
        error: (err) => {
          console.error('Erreur récupération utilisateurs partagés:', err);
          this.sharedUsers = [];
        },
      });
    }
  }

  loadUsername(userId: string) {
    this.userService.getUserById(userId).subscribe({
      next: (user) => {
        this.username = user.username;
      },
      error: (err) => {
        console.error('Erreur lors de la récupération du pseudo :', err);
        this.username = 'Utilisateur inconnu';
      },
    });
  }

  loadImages() {
    this.pin.files.forEach((imageId) => {
      this.imageService.getImage(imageId).subscribe((blob) => {
        const objectUrl = URL.createObjectURL(blob);
        const safeUrl = this.sanitizer.bypassSecurityTrustUrl(objectUrl);
        this.imageUrls.push(safeUrl);
        if (this.imageUrls.length === this.pin.files.length) {
          this.imagesLoaded = true;
        }
      });
    });
  }

  loadSharedUsers(pinId: string) {
    this.pinService.getSharedUsersForPin(pinId).subscribe({
      next: (shares) => {
        this.sharedUsers = shares;
      },
      error: (err) => {
        console.error('Erreur récupération utilisateurs partagés:', err);
        this.sharedUsers = [];
      },
    });
  }

  ngAfterViewInit(): void {
    this.checkIfDescriptionIsTruncated();
  }

  toggleDescription() {
    this.expandedDescription = !this.expandedDescription;
    if (!this.expandedDescription) {
      // re-check quand on replie
      setTimeout(() => this.checkIfDescriptionIsTruncated(), 0);
    }
  }

  checkIfDescriptionIsTruncated() {
    if (!this.descriptionDiv) return;

    const el = this.descriptionDiv.nativeElement;
    this.showToggleButton = el.scrollHeight > el.clientHeight;
  }

  prevSlide(): void {
    this.currentIndex =
      (this.currentIndex - 1 + this.imageUrls.length) % this.imageUrls.length;
  }

  nextSlide(): void {
    this.currentIndex = (this.currentIndex + 1) % this.imageUrls.length;
  }

  goBack() {
    this.router.navigate(['/map'], { queryParams: { pin: this.pin.id } });
  }
}
