import { CommonModule, NgIf } from '@angular/common';
import { Component, EventEmitter, Input, Output } from '@angular/core';
import { DomSanitizer, SafeUrl } from '@angular/platform-browser';
import * as L from 'leaflet';
import { Pin } from '../../model/Pin';
import { ImageService } from '../../services/image/image.service';
import { ModalService } from '../../services/modal/modal.service';
import { PinService } from '../../services/pin/pin.service';
import { ConfirmModalComponent } from '../confirm-modal/confirm-modal.component';
import { EditPinPopupComponent } from '../edit-pin-popup/edit-pin-popup.component';
import { ShareModalComponent } from '../share-modal/share-modal.component';
import { Router } from '@angular/router';
import { RouterModule } from '@angular/router';

@Component({
  selector: 'app-pin-marker',
  templateUrl: './pin-marker.component.html',
  imports: [
    CommonModule,
    EditPinPopupComponent,
    ConfirmModalComponent,
    ShareModalComponent,
    RouterModule,
    NgIf
  ],
  standalone: true,
})
export class PinMarkerComponent {
  @Input() pin!: Pin;
  @Input() marker!: L.Marker;

  currentIndex: number = 0;
  imageUrls: SafeUrl[] = [];
  imagesLoaded = false;

  @Output() pinOpened = new EventEmitter<void>();

  constructor(
    private pinService: PinService,
    private modalService: ModalService,
    private imageService: ImageService,
    private sanitizer: DomSanitizer,
    private router: Router
  ) {}

  ngOnInit() {
    // Écouter l'événement d'ouverture du popup
    this.marker.on('popupopen', () => {
      if (!this.imagesLoaded) {
        this.loadImages();
        this.pinOpened.emit();
      }
    });
  }

  loadImages() {
    this.pin.files.forEach((imageId) => {
      this.imageService.getImage(imageId).subscribe((blob) => {
        const objectUrl = URL.createObjectURL(blob);
        const safeUrl = this.sanitizer.bypassSecurityTrustUrl(objectUrl);
        this.imageUrls.push(safeUrl);
        if (this.imageUrls.length === this.pin.files.length) {
          this.imagesLoaded = true;
        }
      });
    });
  }

  sharePin() {
    this.modalService.openModal('share-modal-' + this.pin.id);
  }

  onClosePopup() {
    this.marker.closePopup();
  }

  onDelete() {
    this.modalService.openModal('confirm-modal-' + this.pin.id);
  }

  handleConfirm() {
    this.pinService.deletePin(this.pin.id).subscribe(() => {
      this.marker.remove();
      this.modalService.closeModal('confirm-modal-' + this.pin.id);
    });
  }

  handleCancel() {
    this.modalService.closeModal('confirm-modal-' + this.pin.id);
  }

  prevSlide(): void {
    this.currentIndex =
      (this.currentIndex - 1 + this.imageUrls.length) % this.imageUrls.length;
  }

  nextSlide(): void {
    this.currentIndex = (this.currentIndex + 1) % this.imageUrls.length;
  }

  ngOnDestroy() {
    // Clean up object URLs to prevent memory leaks
    this.imageUrls.forEach((url) => {
      URL.revokeObjectURL(url.toString());
    });
  }
}
