import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable, tap } from 'rxjs';
import { environment } from '../../../environment';
import { AuthResponse } from '../../model/AuthResponse';
import { CookiesService } from '../cookies/cookies.service';
import { PushService } from '../push/push.service';
import { Router } from '@angular/router';

@Injectable({
  providedIn: 'root',
})
export class AuthService {
  private isAdminSubject = new BehaviorSubject<boolean>(false);
  private userIdSubject = new BehaviorSubject<string>('');
  isAdmin$ = this.isAdminSubject.asObservable();
  username$ = new BehaviorSubject<string>('');
  isLoggedIn$ = new BehaviorSubject<boolean>(false);
  userId$ = this.userIdSubject.asObservable();

  constructor(
    private http: HttpClient,
    private cookiesService: CookiesService,
    private pushService: PushService,
    private router: Router
  ) {
    const token = this.cookiesService.getToken();
    if (token) {
      this.isAdminSubject.next(this.cookiesService.getIsAdmin() === 'true');
      this.username$.next(this.cookiesService.getUsername() || '');
      this.userIdSubject.next(this.cookiesService.getUserId() || '');
      this.isLoggedIn$.next(true);
    }
  }

  login(username: string, password: string): Observable<AuthResponse> {
    const payload = new HttpParams()
      .set('username', username)
      .set('password', password);

    return this.http
      .post<AuthResponse>(`${environment.apiURL}/login`, payload)
      .pipe(
        tap((response) => {
          this.cookiesService.setToken(response.access_token);
          this.cookiesService.setRefreshToken(response.refresh_token);
          this.cookiesService.setIsAdmin(response.is_admin);
          this.cookiesService.setUsername(username);
          this.cookiesService.setUserId(response.user_id);
          this.isAdminSubject.next(response.is_admin);
          this.username$.next(username);
          this.userIdSubject.next(response.user_id);
          this.isLoggedIn$.next(true);
          this.pushService.enableNotifications();
        })
      );
  }

  logout(): void {
    this.pushService.unsubscribe().finally(() => {
      this.isAdminSubject.next(false);
      this.username$.next('');
      this.userIdSubject.next('');
      this.isLoggedIn$.next(false);
      this.cookiesService.clearSession();
      this.router.navigate(['/']);
    });
  }

  register(username: string, password: string): Observable<AuthResponse> {
    return this.http
      .post<AuthResponse>(`${environment.apiURL}/register`, {
        username,
        password,
      })
      .pipe(
        tap((response) => {
          this.cookiesService.setToken(response.access_token);
          this.cookiesService.setRefreshToken(response.refresh_token);
          this.cookiesService.setIsAdmin(response.is_admin);
          this.cookiesService.setUsername(username);
          this.cookiesService.setUserId(response.user_id);
          this.isAdminSubject.next(response.is_admin);
          this.username$.next(username);
          this.userIdSubject.next(response.user_id);
          this.isLoggedIn$.next(true);
          this.pushService.enableNotifications();
        })
      );
  }

  isAdmin(): boolean {
    return this.isAdminSubject.value || false;
  }

  getUsername(): string {
    return this.username$.value || '';
  }

  getUserId(): string {
    return this.userIdSubject.value || '';
  }

  isLoggedIn(): boolean {
    return this.isLoggedIn$.value || false;
  }
}
