import { Injectable } from '@angular/core';
import { CookieService } from 'ngx-cookie-service';
import { tap, firstValueFrom } from 'rxjs';
import { AuthResponse } from '../../model/AuthResponse';
import { environment } from '../../../environment';
import { HttpClient } from '@angular/common/http';

@Injectable({
  providedIn: 'root',
})
export class CookiesService {
  private readonly AUTH_TOKEN_KEY = 'auth_token';
  private readonly USERNAME_KEY = 'username';
  private readonly USER_ID = 'userId';
  private readonly IS_ADMIN_KEY = 'isAdmin';
  private readonly REFRESH_TOKEN_KEY = 'refresh_token';

  private get COOKIE_OPTIONS() {
    const isLocalhost = window.location.hostname === 'localhost' || 
                       window.location.hostname === '127.0.0.1' ||
                       window.location.hostname.includes('localhost');

    // Détecte le domaine principal pour inclure tous les sous-domaines
    let domain = window.location.hostname;
    if (!isLocalhost) {
      // Prend uniquement le domaine principal (ex: .memorymap.fr)
      const parts = domain.split('.');
      if (parts.length > 2) {
        domain = '.' + parts.slice(-2).join('.');
      } else {
        domain = '.' + domain;
      }
    }

    return {
      path: '/',
      domain: domain,
      secure: !isLocalhost,
      sameSite: 'Strict' as const,
    };
  }

  constructor(
    private cookieService: CookieService,
    private http: HttpClient
  ) {}

  setToken(token: string): void {
    this.cookieService.set(this.AUTH_TOKEN_KEY, token, this.COOKIE_OPTIONS);
  }

  setRefreshToken(refresh_token: string): void {
    this.cookieService.set(this.REFRESH_TOKEN_KEY, refresh_token, this.COOKIE_OPTIONS);
  }

  setUsername(username: string): void {
    this.cookieService.set(this.USERNAME_KEY, username, this.COOKIE_OPTIONS);
  }

  setUserId(user_id: string): void {
    this.cookieService.set(this.USER_ID, user_id, this.COOKIE_OPTIONS);
  }

  getUserId(): string | null {
    return this.cookieService.get(this.USER_ID) || null;
  }

  getUsername(): string | null {
    return this.cookieService.get(this.USERNAME_KEY) || null;
  }

  removeUsername(): void {
    this.cookieService.delete(
      this.USERNAME_KEY,
      this.COOKIE_OPTIONS.path,
      this.COOKIE_OPTIONS.domain
    );
  }

  getToken(): string | null {
    return this.cookieService.get(this.AUTH_TOKEN_KEY);
  }

  clearSession() {
    this.removeToken();
    this.removeRefreshToken();
    this.removeIsAdmin();
    this.removeUsername();
    this.removeUserId();
  }

  getRefreshToken(): string | null {
    return this.cookieService.get(this.REFRESH_TOKEN_KEY) || null;
  }

  async getValidToken(): Promise<string | null> {
    const token = this.cookieService.get(this.AUTH_TOKEN_KEY);
    if (!token) return null;
  
    const payload = JSON.parse(atob(token.split('.')[1]));
    const expirationDate = new Date(payload.exp * 1000);
  
    if (expirationDate < new Date()) {
      try {
        const refreshedResponse = await this.tryRefreshToken();
        return refreshedResponse ? this.cookieService.get(this.AUTH_TOKEN_KEY) : null;
      } catch (e) {
        console.error('Erreur lors du refresh token', e);
        return null;
      }
    }
  
    return token;
  }

  async tryRefreshToken(): Promise<boolean> {
    const refreshToken = this.getRefreshToken();
    if (!refreshToken) return false;

    try {
      const response = await firstValueFrom(
        this.http.post<AuthResponse>(`${environment.apiURL}/refresh-token`, {
          refresh_token: refreshToken
        }).pipe(
          tap((response) => {
            this.setToken(response.access_token);
            this.setRefreshToken(response.refresh_token);
            this.setIsAdmin(response.is_admin);
            this.setUserId(response.user_id);
          })
        )
      );
      
      return !!response;
    } catch (error) {
      console.error('Erreur lors du refresh token', error);
      return false;
    }
  }

  removeToken(): void {
    this.cookieService.delete(
      this.AUTH_TOKEN_KEY,
      this.COOKIE_OPTIONS.path,
      this.COOKIE_OPTIONS.domain
    );
  }

  removeRefreshToken(): void {
    this.cookieService.delete(
      this.REFRESH_TOKEN_KEY,
      this.COOKIE_OPTIONS.path,
      this.COOKIE_OPTIONS.domain
    );
  }

  removeUserId(): void {
    this.cookieService.delete(
      this.USER_ID,
      this.COOKIE_OPTIONS.path,
      this.COOKIE_OPTIONS.domain
    );
  }

  setIsAdmin(isAdmin: boolean): void {
    this.cookieService.set(
      this.IS_ADMIN_KEY,
      isAdmin.toString(),
      this.COOKIE_OPTIONS
    );
  }

  getIsAdmin(): string | null {
    return this.cookieService.get(this.IS_ADMIN_KEY) || null;
  }

  removeIsAdmin(): void {
    this.cookieService.delete(
      this.IS_ADMIN_KEY,
      this.COOKIE_OPTIONS.path,
      this.COOKIE_OPTIONS.domain
    );
  }
}
